

{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (garetxe@gmail.com)

Encapsulates information about a key event.
-}

module GI.Atk.Structs.KeyEventStruct
    ( 

-- * Exported types
    KeyEventStruct(..)                      ,
    newZeroKeyEventStruct                   ,
    noKeyEventStruct                        ,


 -- * Properties
-- ** Keycode
    keyEventStructReadKeycode               ,


-- ** Keyval
    keyEventStructReadKeyval                ,


-- ** Length
    keyEventStructReadLength                ,


-- ** State
    keyEventStructReadState                 ,


-- ** String
    keyEventStructReadString                ,


-- ** Timestamp
    keyEventStructReadTimestamp             ,


-- ** Type
    keyEventStructReadType                  ,




    ) where

import Prelude ()
import Data.GI.Base.ShortPrelude

import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map

import GI.Atk.Types
import GI.Atk.Callbacks

newtype KeyEventStruct = KeyEventStruct (ForeignPtr KeyEventStruct)
-- | Construct a `KeyEventStruct` struct initialized to zero.
newZeroKeyEventStruct :: MonadIO m => m KeyEventStruct
newZeroKeyEventStruct = liftIO $ callocBytes 32 >>= wrapPtr KeyEventStruct

noKeyEventStruct :: Maybe KeyEventStruct
noKeyEventStruct = Nothing

keyEventStructReadType :: KeyEventStruct -> IO Int32
keyEventStructReadType s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 0) :: IO Int32
    return val

keyEventStructReadState :: KeyEventStruct -> IO Word32
keyEventStructReadState s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 4) :: IO Word32
    return val

keyEventStructReadKeyval :: KeyEventStruct -> IO Word32
keyEventStructReadKeyval s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 8) :: IO Word32
    return val

keyEventStructReadLength :: KeyEventStruct -> IO Int32
keyEventStructReadLength s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 12) :: IO Int32
    return val

keyEventStructReadString :: KeyEventStruct -> IO T.Text
keyEventStructReadString s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 16) :: IO CString
    val' <- cstringToText val
    return val'

keyEventStructReadKeycode :: KeyEventStruct -> IO Word16
keyEventStructReadKeycode s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 24) :: IO Word16
    return val

keyEventStructReadTimestamp :: KeyEventStruct -> IO Word32
keyEventStructReadTimestamp s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 28) :: IO Word32
    return val

type family ResolveKeyEventStructMethod (t :: Symbol) (o :: *) :: * where
    ResolveKeyEventStructMethod l o = MethodResolutionFailed l o

instance (info ~ ResolveKeyEventStructMethod t KeyEventStruct, MethodInfo info KeyEventStruct p) => IsLabelProxy t (KeyEventStruct -> p) where
    fromLabelProxy _ = overloadedMethod (MethodProxy :: MethodProxy info)

#if MIN_VERSION_base(4,9,0)
instance (info ~ ResolveKeyEventStructMethod t KeyEventStruct, MethodInfo info KeyEventStruct p) => IsLabel t (KeyEventStruct -> p) where
    fromLabel _ = overloadedMethod (MethodProxy :: MethodProxy info)
#endif


